﻿using System;
using System.IO;
using System.Threading.Tasks;
using BeSafePlus.Models;
using SQLite;
using Microsoft.Extensions.Logging;
using Microsoft.Maui.Storage;
using BeSafePlus.Sensors;
using System.Diagnostics;

namespace BeSafePlus.Database
{
    public class LocalDatabase : ILocalDatabase
    {
        private readonly SQLiteAsyncConnection _connection;

        private readonly ILogger<LocalDatabase> _logger;
        public LocalDatabase(ILogger<LocalDatabase> logger)
        {
            _logger = logger;

            string databasePath = Path.Combine(FileSystem.AppDataDirectory, "BeSafePlus.db");

            try
            {
                _connection = new SQLiteAsyncConnection(databasePath);

                Task.Run(Initialise).GetAwaiter().GetResult();
            }

            catch (Exception ex)
            {
                _logger.LogError(ex, "Database initialization failed");
            }
        }
        private async Task Initialise()
        {
            try
            {
                await Task.WhenAll(
                 _connection.CreateTableAsync<StepAlert>(),
                 _connection.CreateTableAsync<User>(),
                 _connection.CreateTableAsync<StepCount>(),
                 _connection.CreateTableAsync<ActivityLevel>(),
                 _connection.CreateTableAsync<RedCapToken>());
            }

            catch (Exception ex)
            {
                _logger.LogError(ex, "Table creation failed");
            }
        }
        public async Task UpdateUser(User user)
        {
            var existingUser = await _connection.Table<User>().FirstOrDefaultAsync();
            if(existingUser != null)
            {
                user.Id = existingUser.Id;
                await _connection.UpdateAsync(user);
            }
        }
        public async Task SetUserId()
        {
            var id = DateTimeOffset.Now.ToUnixTimeMilliseconds();

            var userId = new User
            {
                Id = id,
            };
            await _connection.InsertAsync(userId);
        }
        public async Task<User?> GetUser()
        {
            return await _connection.Table<User>().FirstOrDefaultAsync();
        }
        public async Task<int> UpdateUserAttribute(Func<User, object> getValue, Action<User, object> setValue, object newValue)
        {
            var existingUser = await _connection.Table<User>().FirstOrDefaultAsync();

            if (existingUser != null)
            {
                setValue(existingUser, newValue);

                return await _connection.UpdateAsync(existingUser);
            }
            return 0;
        }
        public Task UpdateUserName(string newName) => UpdateUserAttribute(u => u.Name, (u, v) => u.Name = (string)v, newName);
        public Task UpdateUserGender(string newGender) => UpdateUserAttribute(u => u.Gender, (u, v) => u.Gender = (string)v, newGender);
        public Task UpdateUserAge(int newAge) => UpdateUserAttribute(u => u.Age, (u, v) => u.Age = (int)v, newAge);
        public Task UpdateUserHeight(int newHeight) => UpdateUserAttribute(u => u.Height, (u, v) => u.Height = (int)v, newHeight);
        public Task UpdateUserWeight(string newWeight) => UpdateUserAttribute(u => u.Weight, (u, v) => u.Weight = (string)v, newWeight);
        public async Task<int> CreateOrUpdateStepCount(int steps)
        {
            string dateString = DateTime.Now.ToString("yyyy-MM-dd");

            string previousDateString = DateTime.Now.AddDays(-1).ToString("yyyy-MM-dd");

            var existingRecord = await _connection.Table<StepCount>().FirstOrDefaultAsync(sc => sc.Date == dateString);

            if (existingRecord != null)
            {
                existingRecord.Steps = steps;

                return await _connection.UpdateAsync(existingRecord);
            }
            else
            {
                // Get the last known step record (from the previous day)
                var lastRecord = await _connection.Table<StepCount>()
                    .FirstOrDefaultAsync(sc => sc.Date == previousDateString);

                // Determine the step goal
                int stepGoal = (lastRecord != null && lastRecord.StepGoal.HasValue)
                    ? lastRecord.StepGoal.Value
                    : 5000;

                var newStepCount = new StepCount
                {
                    Date = dateString,
                    Steps = steps,
                    StepGoal = stepGoal
                };

                return await _connection.InsertAsync(newStepCount);
            }
        }
        public async Task<int> CreateStepAlert(StepAlert stepalert)
        {
            stepalert.Date = DateTime.Now.ToString("yyyy-MM-dd");

            return await _connection.InsertAsync(stepalert);
        }
        public async Task<int> DeleteUser(User user)
        {
            return await _connection.DeleteAsync(user);
        }
        public async Task<StepCount?> GetStepCountByDate(DateTime date)
        {
            string dateString = date.ToString("yyyy-MM-dd");

            return await _connection.Table<StepCount>().Where(sc => sc.Date == dateString).FirstOrDefaultAsync();
        }
        public async Task<StepAlert?> GetStepAlertByDate(DateTime date)
        {
            string dateString = date.ToString("yyyy-MM-dd");

            return await _connection.Table<StepAlert>().Where(f => f.Date == dateString).FirstOrDefaultAsync();
        }
        public async Task<int> UpdateStepGoal(int stepGoal)
        {
            string dateString = DateTime.Now.ToString("yyyy-MM-dd");

            var existingRecord = await _connection.Table<StepCount>().FirstOrDefaultAsync(sc => sc.Date == dateString);

            if (existingRecord != null)
            {
                existingRecord.StepGoal = stepGoal;

                return await _connection.UpdateAsync(existingRecord);
            }

            else
            {
                var newStepCount = new StepCount
                {
                    Date = dateString,
                    Steps = 0, // Default to 0 steps
                    StepGoal = stepGoal
                };

                return await _connection.InsertAsync(newStepCount);
            }
        }
        public async Task SaveActivityLevel(SensorEventHandler.UserActivityState userActivityState, DateTime timeOfActivityChange)
        {
            var activityLevel = userActivityState.ToString();

            var timeOfChange = timeOfActivityChange.ToString("yyyy-MM-dd HH:mm");

            await _connection.InsertAsync(new ActivityLevel
            {
                UserActivityLevel = activityLevel,
                Time = timeOfChange
            });

            Debug.WriteLine($"Activity data was added to database: {activityLevel};{timeOfChange}");
        }
        public async Task FlushActivityLevel()
        {
            await _connection.DeleteAllAsync<ActivityLevel>();
        }
        public async Task<string> GetName()
        {
            var user = await _connection.Table<User>().FirstOrDefaultAsync();

            return user?.Name ?? "Unknown";
        }
        public async Task<string> GetGender()
        {
            var user = await _connection.Table<User>().FirstOrDefaultAsync();

            return user?.Gender ?? "Unknown";
        }
        public async Task<int> GetAge()
        {
            var user = await _connection.Table<User>().FirstOrDefaultAsync();

            return user?.Age ?? 0;
        }
        public async Task<int> GetHeight()
        {
            var user = await _connection.Table<User>().FirstOrDefaultAsync();

            return user?.Height ?? 0;
        }
        public async Task<string> GetWeight()
        {
            var user = await _connection.Table<User>().FirstOrDefaultAsync();

            return user?.Weight ?? "Unknown";
        }
        public async Task<List<ActivityLevel>> GetTodaysActivity()
        {
            var response = await _connection.Table<ActivityLevel>().ToListAsync();

            return response;
        }
        public async Task<RedCapToken?> GetRedCapToken()
        {
            return await _connection.Table<RedCapToken>().FirstOrDefaultAsync();
        }
        public async Task SaveRedCapToken(string redCapToken)
        {
            //
        }
    }
}