using UIKit;
using CoreMotion;
using Foundation;
using System;
using System.Diagnostics;
using BeSafePlus.Sensors;
using BeSafePlus.Extensions;

namespace BeSafePlus.Platforms.iOS.Sensor
{
    public class ActivityTrackingSensor : IActivityTrackingSensor
    {
        private CMPedometer _pedometer;

        private CMMotionManager _motionManager;

        private bool _isTracking;

        private int _currentBaseStepCount; // Store the last recorded step count

        private bool _isFirstReading = true; // Flag for the first reading

        private Timer _resetStepCounterTimer;

        public event Action<int, DateTime>? StepCounterChangeEvent;

        public event Action<float, float, float, DateTime>? AccelerometerChangeEvent;

        public event Action? ResetStepCountEvent;
        public ActivityTrackingSensor()
        {
            _pedometer = new CMPedometer();

            _motionManager = new CMMotionManager();

            _isTracking = false;
        }
        public void Start()
        {
            Debug.WriteLine("Start method called.");

            CMAuthorizationStatus authStatus = CMPedometer.AuthorizationStatus;

            if (authStatus == CMAuthorizationStatus.NotDetermined)
            {
                RequestPermission();
            }

            else if (authStatus == CMAuthorizationStatus.Denied)
            {
                Debug.WriteLine("Motion tracking permission denied.");
            }

            else if (authStatus == CMAuthorizationStatus.Restricted)
            {
                Debug.WriteLine("Motion tracking is restricted.");
            }

            else if (authStatus == CMAuthorizationStatus.Authorized)
            {
                StartTracking();
            }

            else
            {
                Debug.WriteLine("Unexpected authorization status.");
            }
        }
        private void RequestPermission()
        {
            _pedometer.QueryPedometerData(NSDate.Now, NSDate.Now, (data, error) =>
            {
                if (error != null)
                {
                    Debug.WriteLine($"Error requesting permission: {error.LocalizedDescription}");

                    return;
                }
                StartTracking();
            });
        }
        public void StartTracking()
        {
            if (!_isTracking)
            {
                _isTracking = true;

                if (CMPedometer.IsStepCountingAvailable)
                {
                    _pedometer.StartPedometerUpdates(NSDate.Now, (data, error) =>
                    {
                        if (error != null)
                        {
                            Debug.WriteLine($"Error: {error.LocalizedDescription}");

                            return;
                        }

                        if (data != null)
                        {
                            int stepsRead = (int)data.NumberOfSteps;

                            if (_isFirstReading)
                            {
                                _isFirstReading = false;

                                _currentBaseStepCount = stepsRead;
                            }

                            int newSteps = stepsRead - _currentBaseStepCount;

                            _currentBaseStepCount = stepsRead;

                            StepCounterChangeEvent?.Invoke(newSteps, DateTime.Now);
                        }
                    });
                }

                if (_motionManager.AccelerometerAvailable)
                {
                    _motionManager.StartAccelerometerUpdates(NSOperationQueue.MainQueue, (data, error) =>
                    {
                        if (error != null)
                        {
                            Debug.WriteLine($"Error: {error.LocalizedDescription}");

                            return;
                        }

                        if (data != null)
                        {
                            AccelerometerChangeEvent?.Invoke(
                                (float)data.Acceleration.X,
                                (float)data.Acceleration.Y,
                                (float)data.Acceleration.Z,
                                DateTime.Now);
                        }
                    });
                }
            }
        }
        public void Stop()
        {
            if (_isTracking)
            {
                _isTracking = false;

                _pedometer.StopPedometerUpdates();

                _motionManager.StopAccelerometerUpdates();
            }
        }

        public void ResetDailySteps()
        {
            ResetStepCountEvent?.Invoke();
        }

    }
}
