using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BeSafePlus.Models;
using BeSafePlus.Database;
using System.Diagnostics;
using BeSafePlus.Sensors;
using SkiaSharp;

namespace BeSafePlus.Sensors
{
    /// <summary>
    /// This class handles sensor events from stepsensor and accelerometer and sends events to update UI on mainthread
    /// </summary>
    public class SensorEventHandler
    {
        private readonly IActivityTrackingSensor _sensor;
        private readonly ILocalDatabase _db;

        public event Action<int>? UserStepEvent;
        public SensorEventHandler(IActivityTrackingSensor trackingSensor, ILocalDatabase database)
        {
            _db = database;
            _sensor = trackingSensor;

            _sensor.StepCounterChangeEvent += OnStepCounterChange;
            _sensor.AccelerometerChangeEvent += OnAccelerometerChange;
            _sensor.ResetStepCountEvent += OnResetDailySteps;

            InitializeDailyStepsAsync();
            SetThresholds();
        }
        #region StepCounter
        public int DailySteps { get; set; }
        private DateTime LastSaveTimeStamp { get; set; } = DateTime.Now;
        public async void OnStepCounterChange(int steps, DateTime timeOfEvent)
        {
            #if IOS
                // Check if the date has changed (only for iOS)
                if (LastSaveTimeStamp.Date < timeOfEvent.Date)
                {
                    OnResetDailySteps();
                }
            #endif
            DailySteps += steps;

            // Invoke event to UI / ViewModel
            SendDailyStepUpdate();

            // Update recorded steps in DB every 10 seconds
            if ((timeOfEvent - LastSaveTimeStamp).TotalSeconds >= 10)
            {
                await _db.CreateOrUpdateStepCount(DailySteps);

                LastSaveTimeStamp = timeOfEvent;
            }
        }
        private void SendDailyStepUpdate()
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                UserStepEvent?.Invoke(DailySteps);
            });
        }
        private async void InitializeDailyStepsAsync()
        {
            var stepCount = await _db.GetStepCountByDate(DateTime.Now);
            DailySteps += stepCount?.Steps ?? 0;
            SendDailyStepUpdate();
        }
        private void OnResetDailySteps()
        {
            DailySteps = 0;
            SendDailyStepUpdate();
            _db.CreateOrUpdateStepCount(DailySteps).Wait();
            LastSaveTimeStamp = DateTime.Now;
        }
        
        #endregion
        #region Accelerometer
        // IN TESTING
        // https://developer.android.com/develop/sensors-and-location/sensors/sensors_motion#sensors-motion-accel
        private const float ALPHA = 0.8f; // adjust based on testing
        // All thresholds are 99% upper limit confidence (avg + (Z * stddev)) ALPHA 0.8
        #region Activity thresholds
        private float PHONE_LAYING_STILL_THRESHOLD; // based on 2000 samples
        private float USING_PHONE_THRESHOLD; // based on 3730 samples
        private float WALKING_WITH_PHONE_IN_HAND_THRESHOLD; // based on 9203 samples
        private float WALKING_THRESHOLD;
        #endregion


        private const int CONSECUTIVE_STATE_LIMIT = 10;
        private float[] _gravity = new float[3];
        private float[] _linear_acceleration = new float[3];
        private bool _changeInActivityLevel = false;
        int counter = 0;
        int stateConsecutiveCounter = 0;
        public enum UserActivityState
        {
            Stationary,
            PhoneInUse,
            Active
        }
        public UserActivityState InternalUserAcitivityState = UserActivityState.PhoneInUse;
        public UserActivityState TrueUserAcitivityState = UserActivityState.PhoneInUse;
        /*

        Collect Baseline Data

    Run the app and log acceleration values when:
        The phone is on a table.
        The phone is in a pocket but the user is sitting.
        The phone is in a pocket while walking.
    Compute the standard deviation or moving average of the acceleration changes.

        */
        public void OnAccelerometerChange(float x, float y, float z, DateTime timeOfEvent)
        {
            // Low pass filter - isolate force of gravity
            _gravity[0] = ALPHA * _gravity[0] + (1 - ALPHA) * x;
            _gravity[1] = ALPHA * _gravity[1] + (1 - ALPHA) * y;
            _gravity[2] = ALPHA * _gravity[2] + (1 - ALPHA) * z;

            // High pass filter - remove gravity contribution
            _linear_acceleration[0] = x - _gravity[0];
            _linear_acceleration[1] = y - _gravity[1];
            _linear_acceleration[2] = z - _gravity[2];

            var acceleration = MathF.Sqrt(_linear_acceleration[0] * _linear_acceleration[0] +
                                            _linear_acceleration[1] * _linear_acceleration[1] +
                                            _linear_acceleration[2] * _linear_acceleration[2]);

            //Debug.WriteLine($"ACCELEROMETER ACCELERATION: {acceleration} | COUNT: {counter++}");

            if (acceleration < PHONE_LAYING_STILL_THRESHOLD)
            {
                if (InternalUserAcitivityState == UserActivityState.Stationary)
                {
                    stateConsecutiveCounter++;
                }
                else
                {
                    stateConsecutiveCounter = 1;
                }
                if (stateConsecutiveCounter == CONSECUTIVE_STATE_LIMIT && TrueUserAcitivityState != UserActivityState.Stationary)
                {
                    Debug.WriteLine("CHANGED TO PHONE ON DESK");
                    TrueUserAcitivityState = UserActivityState.Stationary;
                    _changeInActivityLevel = true;
                }
                InternalUserAcitivityState = UserActivityState.Stationary;
            }
            else if (acceleration >= PHONE_LAYING_STILL_THRESHOLD && acceleration < USING_PHONE_THRESHOLD)
            {
                if (InternalUserAcitivityState == UserActivityState.PhoneInUse)
                {
                    stateConsecutiveCounter++;
                }
                else
                {
                    stateConsecutiveCounter = 1;
                }
                if (stateConsecutiveCounter == CONSECUTIVE_STATE_LIMIT && TrueUserAcitivityState != UserActivityState.PhoneInUse)
                {
                    Debug.WriteLine("CHANGED TO PHONE IN USE STATE");
                    TrueUserAcitivityState = UserActivityState.PhoneInUse;
                    _changeInActivityLevel = true;
                }
                InternalUserAcitivityState = UserActivityState.PhoneInUse;
            }
            else if (acceleration < WALKING_THRESHOLD && acceleration >= USING_PHONE_THRESHOLD)
            {
                if (InternalUserAcitivityState == UserActivityState.Active)
                {
                    stateConsecutiveCounter++;
                }
                else
                {
                    stateConsecutiveCounter = 1;
                }
                if (stateConsecutiveCounter == CONSECUTIVE_STATE_LIMIT && TrueUserAcitivityState != UserActivityState.Active)
                {
                    Debug.WriteLine("CHANGED TO WALKING STATE");
                    TrueUserAcitivityState = UserActivityState.Active;
                    _changeInActivityLevel = true;
                }
                InternalUserAcitivityState = UserActivityState.Active;
            }
            else
            {
                Debug.WriteLine($"PERSON IS RUNNING FOR THEIR LIFE");
            }

            if( _changeInActivityLevel )
            {
                _db.SaveActivityLevel(TrueUserAcitivityState, DateTime.Now);
                _changeInActivityLevel = false;
            }

            TestingTools.LogData(acceleration, "acceleration_log", "csv");
        }

        private void SetThresholds()
        {
#if ANDROID
            PHONE_LAYING_STILL_THRESHOLD = 0.0592f; // based on 2000 samples
            USING_PHONE_THRESHOLD = 1.592091f; // based on 3730 samples
            WALKING_WITH_PHONE_IN_HAND_THRESHOLD = 3.82225f; // based on 9203 samples
            WALKING_THRESHOLD = 15.3f; // 
#endif
#if IOS
            PHONE_LAYING_STILL_THRESHOLD = 0.002107f; // based on 20000 samples
            USING_PHONE_THRESHOLD = 0.030701f; // based on 6000 samples
            WALKING_WITH_PHONE_IN_HAND_THRESHOLD = 0.174276f; // based on 4000 samples
            WALKING_THRESHOLD = 0.0f; // to be done
#endif
        }
        #endregion



    }
}
