﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using System;
using System.Threading.Tasks;
using BeSafePlus.Models;
using BeSafePlus.Database;
using SkiaSharp;
using Microsoft.Maui.Dispatching;
using BeSafePlus.Sensors;
using System.Collections.Generic;
using System.Linq;

namespace BeSafePlus.ViewModels
{
    public partial class ActivityHistoryViewModel : ObservableObject
    {
        private readonly ILocalDatabase _database;

        private readonly SensorEventHandler _sensorEventHandler;

        private List<StepCount> _weeklyStepData;

        private int _currentWeekOffset = 0;

        private int _steps;

        private int _stepGoal = 5000; // Default Daily Goal

        private int _totalStepsThisWeek;
        public ActivityHistoryViewModel(ILocalDatabase database, SensorEventHandler sensorEventHandler)
        {
            _database = database;

            _weeklyStepData = new List<StepCount>();

            _sensorEventHandler = sensorEventHandler;

            _sensorEventHandler.UserStepEvent += UpdateSteps;

            PreviousWeekCommand = new RelayCommand(async () => await PreviousWeek());

            NextWeekCommand = new RelayCommand(async () => await NextWeek(), CanNavigateToNextWeek);
        }

        // Bindable properties
        public int AccumulatedSteps
        {
            get => _steps;
            set
            {
                if (SetProperty(ref _steps, value))
                {
                    OnPropertyChanged(nameof(Progress));
                }
            }
        }
        public float Progress => StepGoal > 0 ? (float) AccumulatedSteps / StepGoal : 0;
        public int StepGoal
        {
            get => _stepGoal;
            set
            {
                if (SetProperty(ref _stepGoal, value))
                {
                    OnPropertyChanged(nameof(Progress));
                }
            }
        }
        private void UpdateSteps(int steps)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                AccumulatedSteps = steps;

                var today = DateTime.Now.Date;

                var todayEntry = WeeklyStepData.FirstOrDefault(data => DateTime.Parse(data.Date).Date == today);

                if (todayEntry == null)
                {
                    WeeklyStepData.Add(new StepCount { Date = today.ToString("yyyy-MM-dd"), Steps = steps });
                }

                else if (todayEntry.Steps != steps)
                {
                    todayEntry.Steps = steps;
                }

                else
                {
                    return;
                }

                TotalStepsThisWeek = WeeklyStepData.Sum(data => data.Steps);

                OnPropertyChanged(nameof(WeeklyStepData));

                OnPropertyChanged(nameof(TotalStepsThisWeek));

                OnPropertyChanged(nameof(AccumulatedSteps));
            });
        }
        public void SetProgress(int steps)
        {
            AccumulatedSteps = steps;
        }
        public async Task LoadStepGoal()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            StepGoal = stepCount?.StepGoal ?? 5000;
        }
        public List<StepCount> WeeklyStepData
        {
            get => _weeklyStepData;
            private set => SetProperty(ref _weeklyStepData, value);
        }
        public int TotalStepsThisWeek
        {
            get => _totalStepsThisWeek;
            set => SetProperty(ref _totalStepsThisWeek, value);
        }
        public string WeekLabel => GetWeekLabel();
        public IRelayCommand PreviousWeekCommand { get; }
        public IRelayCommand NextWeekCommand { get; }
        public bool CanNavigateToNextWeek()
        {
            return _currentWeekOffset < 0;
        }
        public async Task LoadWeeklySteps()
        {
            var today = DateTime.Now.AddDays(_currentWeekOffset * 7);

            var startOfWeek = today.AddDays(-(int)today.DayOfWeek + 1);

            var endOfWeek = startOfWeek.AddDays(6);

            var newWeeklyData = new List<StepCount>();

            int totalSteps = 0;

            for (int i = 0; i < 7; i++)
            {
                var date = startOfWeek.AddDays(i);

                var stepData = await _database.GetStepCountByDate(date);

                if (stepData != null)
                {
                    newWeeklyData.Add(stepData);

                    totalSteps += stepData.Steps;
                }

                else
                {
                    newWeeklyData.Add(new StepCount { Steps = 0, Date = date.ToString("yyyy-MM-dd") });
                }
            }

            WeeklyStepData = newWeeklyData;

            TotalStepsThisWeek = totalSteps;

            OnPropertyChanged(nameof(WeekLabel));

            ((RelayCommand)NextWeekCommand).NotifyCanExecuteChanged();
        }
        private string GetWeekLabel()
        {
            var today = DateTime.Now.AddDays(_currentWeekOffset * 7);

            var startOfWeek = today.AddDays(-(int)today.DayOfWeek + 1);

            var endOfWeek = startOfWeek.AddDays(6);

            return $"{startOfWeek:dd/MM} - {endOfWeek:dd/MM}";
        }
        public async Task PreviousWeek()
        {
            _currentWeekOffset--;

            await LoadWeeklySteps();
        }
        public async Task NextWeek()
        {
            if (_currentWeekOffset < 0)
            {
                _currentWeekOffset++;

                await LoadWeeklySteps();
            }
        }
        public async Task<int> GetStepGoal()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            return stepCount?.StepGoal ?? 5000;
        }
        public async Task<int> GetStepCount()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            return stepCount?.Steps ?? 0;
        }
    }
}